import { ApolloLink, FetchResult } from 'apollo-link-core';
import { DocumentNode } from 'graphql';
import { DataProxy } from 'apollo-cache-core';
import { QueryScheduler } from '../scheduler/scheduler';
import { Observer, Observable } from '../util/Observable';
import { DataStore } from '../data/store';
import { MutationStore } from '../data/mutations';
import { QueryStore } from '../data/queries';
import { WatchQueryOptions, SubscriptionOptions } from './watchQueryOptions';
import { ObservableQuery } from './ObservableQuery';
import { QueryListener, ApolloQueryResult, PureQueryOptions, FetchType, MutationQueryReducersMap } from './types';
export declare class QueryManager {
    pollingTimers: {
        [queryId: string]: any;
    };
    scheduler: QueryScheduler;
    link: ApolloLink;
    ssrMode: boolean;
    mutationStore: MutationStore;
    queryStore: QueryStore;
    dataStore: DataStore;
    private addTypename;
    private deduplicator;
    private queryDeduplication;
    private onBroadcast;
    private queryListeners;
    private queryListenerWatchCancel;
    private queryListenerInvalidated;
    private queryListenerInvalidatedNewData;
    private queryDocuments;
    private idCounter;
    private fetchQueryPromises;
    private observableQueries;
    private queryIdsByName;
    private lastRequestId;
    constructor({link, addTypename, queryDeduplication, ssrMode, store, onBroadcast}: {
        link: ApolloLink;
        addTypename?: boolean;
        queryDeduplication?: boolean;
        ssrMode?: boolean;
        store: DataStore;
        onBroadcast?: () => void;
    });
    mutate<T>({mutation, variables, optimisticResponse, updateQueries: updateQueriesByName, refetchQueries, update: updateWithProxyFn}: {
        mutation: DocumentNode;
        variables?: Object;
        optimisticResponse?: Object | Function;
        updateQueries?: MutationQueryReducersMap<T>;
        refetchQueries?: string[] | PureQueryOptions[];
        update?: (proxy: DataProxy, mutationResult: Object) => void;
    }): Promise<FetchResult<T>>;
    fetchQuery<T>(queryId: string, options: WatchQueryOptions, fetchType?: FetchType, fetchMoreForQueryId?: string): Promise<FetchResult<T>>;
    queryListenerForObserver<T>(queryId: string, options: WatchQueryOptions, observer: Observer<ApolloQueryResult<T>>): QueryListener;
    watchQuery<T>(options: WatchQueryOptions, shouldSubscribe?: boolean): ObservableQuery<T>;
    query<T>(options: WatchQueryOptions): Promise<ApolloQueryResult<T>>;
    generateQueryId(): string;
    stopQueryInStore(queryId: string): void;
    addQueryListener(queryId: string, listener: QueryListener): void;
    updateQueryWatch(queryId: string, options: WatchQueryOptions): void;
    addFetchQueryPromise<T>(requestId: number, promise: Promise<ApolloQueryResult<T>>, resolve: (result: ApolloQueryResult<T>) => void, reject: (error: Error) => void): void;
    removeFetchQueryPromise(requestId: number): void;
    addObservableQuery<T>(queryId: string, observableQuery: ObservableQuery<T>): void;
    removeObservableQuery(queryId: string): void;
    resetStore(): Promise<ApolloQueryResult<any>[]>;
    startQuery<T>(queryId: string, options: WatchQueryOptions, listener: QueryListener): string;
    startGraphQLSubscription(options: SubscriptionOptions): Observable<any>;
    removeQuery(queryId: string): void;
    stopQuery(queryId: string): void;
    getCurrentQueryResult<T>(observableQuery: ObservableQuery<T>): any;
    getQueryWithPreviousResult<T>(queryIdOrObservable: string | ObservableQuery<T>): {
        previousResult: any;
        variables: {
            [key: string]: any;
        } | undefined;
        document: DocumentNode;
    };
    broadcastQueries(): void;
    private getQueryParts<T>(observableQuery);
    private transformQueryDocument(options);
    private fetchRequest<T>({requestId, queryId, document, options, fetchMoreForQueryId});
    private refetchQueryByName(queryName);
    private generateRequestId();
}
