import * as chai from 'chai';
var assert = chai.assert;
import { mockSingleLink, mockObservableLink } from './mocks/mockLinks';
import ApolloClient from '../src';
import gql from 'graphql-tag';
import { ApolloLink } from 'apollo-link-core';
import InMemoryCache from 'apollo-cache-inmemory';
var isSub = function (operation) {
    return operation.query.definitions
        .filter(function (x) { return x.kind === 'OperationDefinition'; })
        .some(function (x) { return x.operation === 'subscription'; });
};
describe('subscribeToMore', function () {
    var query = (_a = ["\n    query aQuery {\n      entry {\n        value\n      }\n    }\n  "], _a.raw = ["\n    query aQuery {\n      entry {\n        value\n      }\n    }\n  "], gql(_a));
    var result = {
        data: {
            entry: {
                value: 1,
            },
        },
    };
    var req1 = { request: { query: query }, result: result };
    var results = ['Dahivat Pandya', 'Amanda Liu'].map(function (name) { return ({
        result: { data: { name: name } },
        delay: 10,
    }); });
    var sub1 = {
        request: {
            query: (_b = ["\n        subscription newValues {\n          name\n        }\n      "], _b.raw = ["\n        subscription newValues {\n          name\n        }\n      "], gql(_b)),
        },
    };
    var results2 = [
        { result: { data: { name: 'Amanda Liu' } }, delay: 10 },
        { error: new Error('You cant touch this'), delay: 10 },
    ];
    var sub2 = {
        request: {
            query: (_c = ["\n        subscription newValues {\n          name\n        }\n      "], _c.raw = ["\n        subscription newValues {\n          name\n        }\n      "], gql(_c)),
        },
    };
    var results3 = [
        { error: new Error('You cant touch this'), delay: 10 },
        { result: { data: { name: 'Amanda Liu' } }, delay: 10 },
    ];
    var sub3 = {
        request: {
            query: (_d = ["\n        subscription newValues {\n          name\n        }\n      "], _d.raw = ["\n        subscription newValues {\n          name\n        }\n      "], gql(_d)),
        },
    };
    it('triggers new result from subscription data', function (done) {
        var latestResult = null;
        var wSLink = mockObservableLink(sub1);
        var httpLink = mockSingleLink(req1);
        var link = ApolloLink.split(isSub, wSLink, httpLink);
        var counter = 0;
        var client = new ApolloClient({
            cache: new InMemoryCache({}, { addTypename: false }),
            link: link,
            addTypename: false,
        });
        var obsHandle = client.watchQuery({ query: query });
        var sub = obsHandle.subscribe({
            next: function (queryResult) {
                latestResult = queryResult;
                counter++;
            },
        });
        obsHandle.subscribeToMore({
            document: (_a = ["\n        subscription newValues {\n          name\n        }\n      "], _a.raw = ["\n        subscription newValues {\n          name\n        }\n      "], gql(_a)),
            updateQuery: function (_, _a) {
                var subscriptionData = _a.subscriptionData;
                return { entry: { value: subscriptionData.data.name } };
            },
        });
        setTimeout(function () {
            sub.unsubscribe();
            assert.equal(counter, 3);
            assert.deepEqual(latestResult, {
                data: { entry: { value: 'Amanda Liu' } },
                loading: false,
                networkStatus: 7,
                stale: false,
            });
            done();
        }, 50);
        for (var i = 0; i < 2; i++) {
            wSLink.simulateResult(results[i]);
        }
        var _a;
    });
    it('calls error callback on error', function (done) {
        var latestResult = null;
        var wSLink = mockObservableLink(sub2);
        var httpLink = mockSingleLink(req1);
        var link = ApolloLink.split(isSub, wSLink, httpLink);
        var counter = 0;
        var client = new ApolloClient({
            link: link,
            addTypename: false,
            cache: new InMemoryCache({}, { addTypename: false }),
        });
        var obsHandle = client.watchQuery({
            query: query,
        });
        var sub = obsHandle.subscribe({
            next: function (queryResult) {
                latestResult = queryResult;
                counter++;
            },
        });
        var errorCount = 0;
        obsHandle.subscribeToMore({
            document: (_a = ["\n        subscription newValues {\n          name\n        }\n      "], _a.raw = ["\n        subscription newValues {\n          name\n        }\n      "], gql(_a)),
            updateQuery: function (_, _a) {
                var subscriptionData = _a.subscriptionData;
                return { entry: { value: subscriptionData.data.name } };
            },
            onError: function () {
                errorCount += 1;
            },
        });
        setTimeout(function () {
            sub.unsubscribe();
            assert.deepEqual(latestResult, {
                data: { entry: { value: 'Amanda Liu' } },
                loading: false,
                networkStatus: 7,
                stale: false,
            });
            assert.equal(counter, 2);
            assert.equal(errorCount, 1);
            done();
        }, 50);
        for (var i = 0; i < 2; i++) {
            wSLink.simulateResult(results2[i]);
        }
        var _a;
    });
    it('prints unhandled subscription errors to the console', function (done) {
        var latestResult = null;
        var wSLink = mockObservableLink(sub3);
        var httpLink = mockSingleLink(req1);
        var link = ApolloLink.split(isSub, wSLink, httpLink);
        var counter = 0;
        var client = new ApolloClient({
            link: link,
            addTypename: false,
            cache: new InMemoryCache({}, { addTypename: false }),
        });
        var obsHandle = client.watchQuery({
            query: query,
        });
        var sub = obsHandle.subscribe({
            next: function (queryResult) {
                latestResult = queryResult;
                counter++;
            },
        });
        var errorCount = 0;
        var consoleErr = console.error;
        console.error = function (_) {
            errorCount += 1;
        };
        obsHandle.subscribeToMore({
            document: (_a = ["\n        subscription newValues {\n          name\n        }\n      "], _a.raw = ["\n        subscription newValues {\n          name\n        }\n      "], gql(_a)),
            updateQuery: function () {
                throw new Error('should not be called because of initial error');
            },
        });
        setTimeout(function () {
            sub.unsubscribe();
            assert.deepEqual(latestResult, {
                data: { entry: { value: 1 } },
                loading: false,
                networkStatus: 7,
                stale: false,
            });
            assert.equal(counter, 1);
            assert.equal(errorCount, 1);
            console.error = consoleErr;
            done();
        }, 50);
        for (var i = 0; i < 2; i++) {
            wSLink.simulateResult(results3[i]);
        }
        var _a;
    });
    var _a, _b, _c, _d;
});
//# sourceMappingURL=subscribeToMore.js.map